#!/usr/bin/perl
use Test::More tests => 16;
use POWER::Epoll;
use Errno;
use Data::Dumper;
use Socket;
no warnings 'once';

$epfd = epoll_create(10);

$PORT = 9750;	# just some unused port

if (not fork) {
    sleep 1;
    socket(C, AF_INET, SOCK_STREAM, getprotobyname('tcp'))  or die "socket: $!";
    fcntl(C,F_SETFL,fcntl(C,F_GETFL,0)|O_NONBLOCK)	    or die "fcntl: $!";
    connect(C, sockaddr_in($PORT, INADDR_LOOPBACK));
    $n = sysread C, $buf, 10;
    die "read: $!" unless defined $n;
    exit;
}

socket(S, AF_INET, SOCK_STREAM, getprotobyname('tcp'))	or die "socket: $!";
setsockopt(S, SOL_SOCKET, SO_REUSEADDR, pack('l', 1))	or die "setsockopt: $!";
bind(S, sockaddr_in($PORT, INADDR_LOOPBACK))		or die "bind: $!";
listen(S, SOMAXCONN)					or die "listen: $!";

accept c, S						or die "accept: $!";
fcntl(c,F_SETFL,fcntl(c,F_GETFL,0)|O_NONBLOCK)		or die "fcntl: $!";
epoll_add($epfd, fileno(c), EPOLLET|EPOLLOUT);

($status, $result) = epoll_wait($epfd, 3);
is($status, 1, '1 fd ready (status)');
is($#$result, 0, '1 fd ready (result)');
is($result->[0][0], fileno(c), 'got c');
is($result->[0][1], EPOLLOUT, 'c = OUT');

($status, $result) = epoll_wait($epfd, 1);
is($status, 0, '0 fd ready (status) -- timeout');
is($#$result, -1, '0 fd ready (result) -- timeout');

$n = syswrite c, " " x 2;
is($n, 2, 'written 2 bytes');
($status, $result) = epoll_wait($epfd, 3);
is($status, 1, '1 fd ready (status)');
is($#$result, 0, '1 fd ready (result)');
is($result->[0][0], fileno(c), 'got c');
is($result->[0][1], EPOLLOUT, 'c = OUT');

$n = syswrite c, " " x 20;
is($n, 20, 'written 20 bytes');
($status, $result) = epoll_wait($epfd, 3);
is($status, 1, '1 fd ready (status)');
is($#$result, 0, '1 fd ready (result)');
is($result->[0][0], fileno(c), 'got c');
is($result->[0][1], EPOLLHUP|EPOLLERR, 'c = HUP|ERR');


#print Dumper [$status, $result];

